# 02/03/22 David Carral - Capturing Homomorphism-Closed Decidable Queries with Existential Rules

**Résumé :** Existential rules are a very popular ontology-mediated query language for which the chase represents a generic computational approach for query answering. It is straightforward that existential rule queries exhibiting chase termination are decidable and can only recognize properties that are preserved under homomorphisms. In this paper, we show the converse: every decidable query that is closed under homomorphism can be expressed by an existential rule set for which the standard chase universally terminates. Membership in this fragment is not decidable, but we show via a diagonalisation argument that this is unavoidable.
**Lien :** <https://hal.archives-ouvertes.fr/lirmm-03345614>

* Focus on the proof techniques.
* Expressivity means "What kind of queries can we express with a given logical language ?"
* Example
  * Take all DBs over a single predicate E. Partition all databases that contain a cycle from those who cannot. Can be done with Datalog.
  * "Even/odd not number of elements" cannot be computed with Datalog.
    * This should not be surprised as this is not even expressible in FO - because it is monotonic.
    * The even/odd query is non-monotonic. Proof: if D|=Even then DU{a(x)} |/= Even. 
* Data complexity. Important because no way to solve problems beyond the data complexity of a language. An upper bound of the problems we can solve with a language. 
  * Datalog does not capture PTIME. 
  * Datalog does not even capture some homomorphism-closed queries \[icalp08 paper dawar,kreutzer]
* Recap data-complexity : 
  * Datalog(P)
  * DisjDat (coNP)
  * ER (re)
  * DisjER (re) 
    * JF : how comes ? 
    * D : 
      * need first a notion of fairness to be sure that a trigger is never forgotten. Then again a fact will be entailed in all branches of the disjunctive chase.
      * also the work presented here introduces a way to transform disjunctive ER into plain ER. So another way of proving it.
* DEF : To show that a query is solvable in polynomial time, one needs a Turing machine that solves the query on any (serialized) database.
* Homomorphism-closed queries. If Q -> D and D->D' then Q->D'. Homomorphism can also rename constants (domain elements).
  * we will be focusing on languages that can capture homomorphism-closed queries.
* Database serialization
* Skolem-chase is as expressive as Datalog.
  * Data-complexity is in P. 
    * Argument :
      * Skolem-chase terminates on any DB IFF it terminates on the critical instance. So WHEN THE CHASE TERMINATES we have a depth for terms (given by the critical instance).
      * Then (KEY PROPERTY) for every database you will have the same depth of terms.
      * Finally, because we consider DATA-COMPLEXITY, all this becomes a polynomial factor (again possibly hiding a bigger constant.)
  * So Why studying the semi-oblivious chase if Datalog is as expressive? (Nice arument!)
    * Because it is undecidable to recognize a set of rules that terminates wrt the (semi-)oblivious chase.
    * So you need for instance acylicity conditions to know semi-oblivious chase termination.
    * Then you will know that your program is also expressive in datalog.
* In a previous paper of Markus (Power of the terminating chase) showed that the data-complexity was beyond elementary. 
  * But this does not solve the question of expressivity.
* Disjunctive Datalog. What is the expressivity ? Datalog is strictly within homomorphism closed queries.  
* (from another work) Example existance of datalog-first strategy that does not terminate, while a terminating sequence exists (which is of course not datalog-first). 
* Main message : if one builds a language based on entailment it cannot go beyond homomorphism closed queries.
* Serialization problem. Technique from Sebastian and Michael to generate all possible string encodings. They use disjunction.

Expressivity of ruleSETs that TERMINATE wrt a specific chase variant.

Hom-closed decidable queries can be expressed by disjunctive existential rules.

1. We start by only considering disj ex rules
2. We then restrict to the standard-chase termination
3. We finally get rid of disjunction


### Encode Turing Machines with Rules

Again queries are abstract queries that partition the set of possible databases.

We have in the hypothesis the fact that the query is decidable.

So we have a Turing machine that computes the query.

"Standard" transformation. We get a ruleset that terminates on the input encoding if and only if the TM terminates.

Question JF : can we catch all wrong encoding of the initial work?  David : open issue but hope that it can be done


### Database serialization

There is a gap to cover from databases (no order) to TM (with order).

Disjunctive rules to guess an order 

elem(x) ^ elem(y) -> (x==y) or (x!=y)

(x!=y) --> (x < y) or (x > y)

(and more rules)

These rule give also incorrect orders, BUT for sure they will give us a CORRECT linear order.

The same thing also works for the properites. We do not which hold and then we guess them.

Third step is that we unfold the order of a tree, where each branch is a subset of the order.

The fourth step is to use a binary encoding for each element of the database, because the TM has a finite vocabulary.


### Core chase termination

Guessing an order which is cyclic may be problematic : it may give an infinite serialization

### Encoding disjunction with Existential Rules

The paper assumes a split on the rulebase (Lemma 18).

This may be generalizable to remove this hypothesis. 

But still remark that implementation-wise this may be suboptimal because the goal of disjunctive reasoning is to go smartly through the branches of the chase, witout materializing everything (exponential blowup). 

